/* 
 
 The basic software used for controling the printer using Gcode stored on a SD card
 inset the SD card with the Gcode stored in print.txt
 Any other file name will result in the software not being able to find the Gcode
 Reset the printer after removing and reinserting the SD card.
 
 Hardware:
 * 74HC165 shift input register atached to pins 42, 40 and 44 of arduino, as detailed below.
 * -Motor coils of the feed and build hopper motors attached to the shift register.
 * -LCD 4x20 screen attached to pins 31, 29, 27, 25, 23 and 17.
 * -Backlight of the LCD screen attached to pin 16.
 * -Feed endstop attached to pin 19.
 * -Build endstop attached to pin 18.
 * -X endstop attached to pin 3.
 * -Y endstop attached to pin 14.
 * -X driver attached to pins 46(stp), 48(dir) and A8(en).
 * -Y driver attached to pins A6(stp), A7(dir) and A2(en).
 x -B driver attached to pins 26(stp), 28(dir) and 24(en).
 x -F driver attached to pins 36(stp), 34(dir) and 30(en).
 * -laser mosfet connected to pin 10.
 
 */

#include <LiquidCrystal.h>
#include <SD.h>

const int chipSelect = 53;

File myFile;

LiquidCrystal lcd(31, 29, 27, 25, 23, 17);
const boolean Backlight = 16;
const boolean F_end = 19;
const boolean B_end = 18;
const boolean X_end = 3;
const boolean Y_end = 14;

const boolean laser = 10;
const byte motorA = 4; //not used without a roller
const byte motorB = 6; //not used without a roller

boolean X_step = 46; //on the Z driver
boolean X_dir = 48;
boolean X_enable = A8;

boolean Y_step = A6; //on the Y driver
boolean Y_dir = A7;
boolean Y_enable = A2;

boolean B_step = 26; //on the E0 driver
boolean B_dir = 28;
boolean B_enable = 24;

boolean F_step = 36; //on the E1 driver
boolean F_dir = 34;
boolean F_enable = 30;


//Build/Feed stepper control variables
int F_speed = 0;
int B_speed = 0;

// movement variables
long X_pos_steps;
long Y_pos_steps;
int X_pos_10um;
int Y_pos_10um;
int Z_pos_10um;
int E_pos_10um;
boolean target_reached = 1;
boolean homed = 0;
boolean Laser = 0;

unsigned long X_speed;
unsigned long Y_speed;

int X_target_steps;
int Y_target_steps;

boolean X_direction;
boolean Y_direction;

boolean X_stepped = 0;
boolean Y_stepped = 0;

// piston movement variabelen
int Build_speed = 0;
int Feed_speed = 0;

// variables for lcd (try to place to interface tab
byte menutab = 0;
byte menuhistory; 
byte menu1tab = 0;
boolean menu1tab_pushed = 0;
boolean anyswitch = 0;

byte printing = 0;
boolean SDfound = 0;

//variables for reading SD card
boolean newSDline = 0;
const byte buffersize = 10; //the size of the Gcode buffer
int Gcodebuffer[buffersize][6];
int bufferloc = 0;
int bufferread = 0;

//laser variables
boolean laseron = 0;
boolean laser_out_of_print_range = 0;


// Pin connected to latch pin (  ) of 74HB165
int latchPin = 42;
// Pin connected to clock pin (CP) of 74HC165
int dataPin = 40;
// Pin connected to clock pin (  ) of 74HC165
int clockPin = 44;





byte switchVar1 = 0;
boolean switchUp, switchDown, switchLeft, switchRight, switchEnter, switchMenu1, switchMenu2, switchMenu3;

void setup() 
{  
  lcd.begin(20, 4);

  pinMode(latchPin, OUTPUT);
  pinMode(clockPin, OUTPUT); 
  pinMode(dataPin, INPUT);

  pinMode(Backlight, OUTPUT);
  digitalWrite(Backlight,0);

  pinMode(F_end, INPUT);
  pinMode(B_end, INPUT);
  pinMode(X_end, INPUT);
  pinMode(Y_end, INPUT);

  pinMode(X_step, OUTPUT);
  pinMode(X_dir, OUTPUT);
  digitalWrite(X_enable, 0);

  pinMode(Y_step, OUTPUT);
  pinMode(Y_dir, OUTPUT);
  digitalWrite(Y_enable, 0);
  
  pinMode(B_step, OUTPUT);
  pinMode(B_dir, OUTPUT);
  digitalWrite(B_enable, 0);

  pinMode(F_step, OUTPUT);
  pinMode(F_dir, OUTPUT);
  digitalWrite(F_enable, 0);
  
  pinMode(laser, OUTPUT);
  pinMode(motorA, OUTPUT);
  pinMode(motorB, OUTPUT);

  pinMode(53, OUTPUT);

  pinMode(X_enable, INPUT);
  pinMode(Y_enable,INPUT);
  
  digitalWrite(motorA, 1);
  digitalWrite(motorB, 1);


}

void loop()
{
  update_movements();
  update_interface();
  update_piston();
  update_backlight();
}




















